@@ -11,10 +11,13 @@
 #' @param nbins the number of bins used when calculating density
 #' (has little effect with quasirandom/random distribution)
 #' @param dodge.width Amount by which points from different aesthetic groups 
-#' will be dodged. This requires that one of the aesthetics is a factor.
+#' will be dodged. This requires that one of the aesthetics is a factor. 
+#' To disable dodging between groups, set this to NULL.
 #' @param na.rm if FALSE, the default, missing values are removed with a warning.
 #' If TRUE, missing values are silently removed.
-#' @param groupOnX `r lifecycle::badge("deprecated")` No longer needed.
+#' @param orientation The orientation (i.e., which axis to group on) is inferred from the data.
+#' This can be overridden by setting `orientation` to either `"x"` or `"y"`.
+#' @param groupOnX `r lifecycle::badge("superseded")` See `orientation`.
 #' @importFrom vipor offsetSingleGroup
 #' @export
 #' @seealso [vipor::offsetSingleGroup()], [geom_quasirandom()]
@@ -25,6 +28,7 @@ position_quasirandom <- function(
   bandwidth = .5,
   nbins = NULL,
   dodge.width = 0,
+  orientation = NULL,
   groupOnX = NULL,
   na.rm = FALSE
 ) {
@@ -33,10 +37,24 @@ position_quasirandom <- function(
   if (!missing(groupOnX)) {
     lifecycle::deprecate_soft(
       when = "0.7.1", what = "position_quasirandom(groupOnX)", 
-      details='ggplot2 now handles this case automatically.'
+      details='The axis to group on is now guessed from the data. To override, specify orientation="x" or "y".'
     )
+    if (groupOnX) {
+      orientation = "x"
+    } else {
+      orientation = "y"
+    }
   }
   
+  if (!method %in% c("quasirandom", "pseudorandom", "smiley", "maxout", "frowney", "minout", "tukey", "tukeyDense")) {
+    cli::cli_abort("{.fn method} must be one of: quasirandom, pseudorandom, smiley, maxout, frowney, minout, tukey, or tukeyDense.")
+  }
+  
+  if (!is.null(orientation) && !(orientation %in% c("x", "y"))) {
+    cli::cli_abort("{.fn orientation} must be 'x', 'y', or NULL.")
+  }
+  
+  
   ggproto(NULL, PositionQuasirandom, 
           width = width, 
           varwidth = varwidth, 
@@ -44,38 +62,76 @@ position_quasirandom <- function(
           nbins = nbins,
           method = method,
           dodge.width = dodge.width,
-          na.rm = na.rm
+          na.rm = na.rm,
+          orientation = orientation
   )
 }
 
 PositionQuasirandom <- ggplot2::ggproto("PositionQuasirandom", Position,
                                         required_aes = c('x', 'y'),
                                         setup_params = function(self, data) {
-                                          flipped_aes = has_flipped_aes(data)
-                                          data <- flip_data(data, flipped_aes)
                                           
-                                          # get the number of points in each x axis group
-                                          # and find the largest group
-                                          max.length <- max(data.frame(table(data$x))$Freq)
-                                          
-                                          list(
+                                          params <- list(
                                             width = self$width,
                                             varwidth = self$varwidth,
                                             bandwidth = self$bandwidth,
                                             nbins = self$nbins,
                                             method = self$method,
-                                            # groupOnX = self$groupOnX, deprecated
                                             dodge.width = self$dodge.width,
                                             na.rm = self$na.rm,
-                                            max.length = max.length,
-                                            flipped_aes = flipped_aes
+                                            orientation = self$orientation
                                           )
+                                          
+                                          if (!is.null(params$orientation)) {
+                                            flipped_aes <- has_flipped_aes(data, params, ambiguous = TRUE)                                                                                      
+                                          } else {
+                                            flipped_aes <- has_flipped_aes(data, group_has_equal = TRUE)
+                                            if (flipped_aes) {
+                                              cli::cli_inform("Orientation inferred to be along y-axis; override with `position_quasirandom(orientation = 'x')`")
+                                            }
+                                          }
+
+                                          params$flipped_aes <- flipped_aes
+                                          data <- flip_data(data, params$flipped_aes)
+                                          
+                                          # get the number of points in each x axis group
+                                          # and find the largest group
+                                          params$max_length <- max(data.frame(table(data$x))$Freq)
+                                          
+                                          # check that the number of groups < number of data points
+                                          if (!anyDuplicated(data$group)) {
+                                            if (!is.null(params$dodge.width)) {
+                                              # Warn if dodge.width was set to something besides default
+                                              if (params$dodge.width != 0) {
+                                                cli::cli_inform(
+                                                  "Each group consists of only one observation; resetting dodge.width to NULL.",
+                                                  "Disable this message by explicitly setting `dodge.width=NULL`, or by adjusting the group aesthetic."
+                                                )
+                                              }
+                                              params$dodge.width = NULL
+                                            }
+
+
+                                          }
+                                          
+                                          params
+                                        },
+                                        
+                                        setup_data = function(self, data, params) {
+
+                                          data <- flip_data(data, params$flipped_aes)
+                                          data <- remove_missing(
+                                            data,
+                                            na.rm = params$na.rm,
+                                            vars = "y",
+                                            name = "position_quasirandom"
+                                          )
+                                          flip_data(data, params$flipped_aes)
                                         },
                                         
                                         compute_panel = function(data, params, scales) {
-                                          data <- ggplot2::remove_missing(data, na.rm = as.logical(params$na.rm))
                                           data <- flip_data(data, params$flipped_aes)
-                                          
+
                                           # perform dodging if necessary
                                           data <- ggplot2:::collide(
                                             data,
@@ -91,7 +147,7 @@ PositionQuasirandom <- ggplot2::ggproto("PositionQuasirandom", Position,
                                             params$width <- ggplot2::resolution(
                                               data$x, zero = FALSE) * 0.4
                                           }
-                                          
+
                                           # split data.frame into list of data.frames
                                           if (!is.null(params$dodge.width)) {
                                             data <- split(data, data$group)
@@ -123,14 +179,14 @@ offset_quasirandom <- function(
   width = 0.4, 
   vary.width = FALSE,
   max.length = NULL,
-  na.rm = FALSE,
+  # na.rm = FALSE,
   ...
 ) {
-  if (any(is.na(data$y))) {
-    if (na.rm) {
-      
-    }
-  }
+  # if (any(is.na(data$y))) {
+  #   if (na.rm) {
+  #     
+  #   }
+  # }
   x.offset <- vipor::aveWithArgs(
     data$y, data$x,
     FUN = vipor::offsetSingleGroup,
